@@ -2,17 +2,30 @@ import itertools
 import queue
 import time
 import weakref
+from typing import List
 
 import redis
 
 from . import _msgs as msgs
-from ._commands import (Int, Float)
+from ._commands import (Int, Float, SUPPORTED_COMMANDS, COMMANDS_WITH_SUB)
 from ._helpers import (
     SimpleError, valid_response_type, SimpleString, NoResponse, casematch,
-    compile_pattern, QUEUED)
+    compile_pattern, QUEUED, encode_command)
+
+
+def _extract_command(fields):
+    cmd = encode_command(fields[0])
+    if cmd in COMMANDS_WITH_SUB and len(fields) >= 2:
+        cmd += ' ' + encode_command(fields[1])
+        cmd_arguments = fields[2:]
+    else:
+        cmd_arguments = fields[1:]
+    return cmd, cmd_arguments
 
 
 class BaseFakeSocket:
+    _connection_error_class = redis.ConnectionError
+
     def __init__(self, server, *args, **kwargs):
         super(BaseFakeSocket, self).__init__(*args, **kwargs)
         self._server = server
@@ -133,7 +146,7 @@ class BaseFakeSocket:
         except SimpleError as exc:
             result = exc
         for command_item in command_items:
-            command_item.writeback()
+            command_item.writeback(remove_empty_val=msgs.FLAG_LEAVE_EMPTY_VAL not in sig.flags)
         return result
 
     def _decode_error(self, error):
@@ -180,17 +193,16 @@ class BaseFakeSocket:
             if ret is not None:
                 return ret
 
-    def _name_to_func(self, name):
-        name = (name.decode(encoding='utf-8', errors='replace')
-                if isinstance(name, bytes)
-                else str(name).encode(encoding='utf-8', errors='replace'))
-        func_name = name.lower()
-        func = getattr(self, func_name, None)
-        if name.startswith('_') or not func or not hasattr(func, '_fakeredis_sig'):
+    def _name_to_func(self, cmd_name: str):
+        """Get the signature and the method from the command name.
+        """
+        if cmd_name not in SUPPORTED_COMMANDS:
             # redis remaps \r or \n in an error to ' ' to make it legal protocol
-            clean_name = name.replace('\r', ' ').replace('\n', ' ')
+            clean_name = cmd_name.replace('\r', ' ').replace('\n', ' ')
             raise SimpleError(msgs.UNKNOWN_COMMAND_MSG.format(clean_name))
-        return func, func_name
+        sig = SUPPORTED_COMMANDS[cmd_name]
+        func = getattr(self, sig.func_name, None)
+        return func, sig
 
     def sendall(self, data):
         if not self._server.connected:
@@ -199,13 +211,13 @@ class BaseFakeSocket:
             data = data.encode('ascii')
         self._parser.send(data)
 
-    def _process_command(self, fields):
+    def _process_command(self, fields: List[bytes]):
         if not fields:
             return
-        func_name = None
+
+        cmd, cmd_arguments = _extract_command(fields)
         try:
-            func, func_name = self._name_to_func(fields[0])
-            sig = func._fakeredis_sig
+            func, sig = self._name_to_func(cmd)
             with self._server.lock:
                 # Clean out old connections
                 while True:
@@ -220,20 +232,18 @@ class BaseFakeSocket:
                 now = time.time()
                 for db in self._server.dbs.values():
                     db.time = now
-                sig.check_arity(fields[1:], self.version)
-                # TODO: make a signature attribute for transactions
-                if self._transaction is not None \
-                        and func_name not in ('exec', 'discard', 'multi', 'watch'):
-                    self._transaction.append((func, sig, fields[1:]))
+                sig.check_arity(cmd_arguments, self.version)
+                if self._transaction is not None and msgs.FLAG_TRANSACTION not in sig.flags:
+                    self._transaction.append((func, sig, cmd_arguments))
                     result = QUEUED
                 else:
-                    result = self._run_command(func, sig, fields[1:], False)
+                    result = self._run_command(func, sig, cmd_arguments, False)
         except SimpleError as exc:
             if self._transaction is not None:
                 # TODO: should not apply if the exception is from _run_command
                 # e.g. watch inside multi
                 self._transaction_failed = True
-            if func_name == 'exec' and exc.value.startswith('ERR '):
+            if cmd == 'exec' and exc.value.startswith('ERR '):
                 exc.value = 'EXECABORT Transaction discarded because of: ' + exc.value[4:]
                 self._transaction = None
                 self._transaction_failed = False
@@ -244,8 +254,7 @@ class BaseFakeSocket:
             self.put_response(result)
 
     def _scan(self, keys, cursor, *args):
-        """
-        This is the basis of most of the ``scan`` methods.
+        """This is the basis of most of the ``scan`` methods.
 
         This implementation is KNOWN to be un-performant, as it requires
         grabbing the full set of keys over which we are investigating subsets.
