@@ -51,6 +51,8 @@ RESTORE_INVALID_CHECKSUM_MSG = "ERR DUMP payload version or checksum are wrong"
 RESTORE_INVALID_TTL_MSG = "ERR Invalid TTL value, must be >= 0"
 JSON_WRONG_REDIS_TYPE = "ERR Existing key has wrong Redis type"
 JSON_KEY_NOT_FOUND = "ERR could not perform this operation on a key that doesn't exist"
+JSON_PATH_NOT_FOUND_OR_NOT_STRING = "ERR Path '{}' does not exist or not a string"
+JSON_PATH_DOES_NOT_EXIST = "ERR Path '{}' does not exist"
 
 FLAG_NO_SCRIPT = 's'  # Command not allowed in scripts
 FLAG_LEAVE_EMPTY_VAL = 'v'
